import nodriver as nd
import time
import random
import asyncio
from bs4 import BeautifulSoup

try:
    from . import general_utilities
    from . import proxies_utilities
    from . import tor_utilities
except:
    import general_utilities
    import proxies_utilities
    import tor_utilities

global hyperSelProxies
hyperSelProxies = None

# Async functions
async def asyc_open_browser(headless, proxy, tor, max_attempts):
    browser = await open_nodriver(headless, proxy, tor, max_attempts)
    return browser

def open_browser(headless=False, proxy=None, tor=False, max_attempts=3):
    return asyncio.run(asyc_open_browser(headless, proxy, tor, max_attempts))

async def async_go_to_site(browser, url):
    page = await browser.get(url=url)
    return page

async def async_get_current_url(page):
    return await page.evaluate("window.location.href")

def get_current_url(page):
    return asyncio.run(async_get_current_url(page))

def go_to_site(browser, url):
    # REUTNRS PAGE
    return asyncio.run(async_go_to_site(browser, url))

async def async_find_best_match(page, tag):
    return await page.find(tag, best_match=True)

def find_best_match(page, tag):
    return asyncio.run(async_find_best_match(page, tag))

async def async_find_nearest_text(page, text):
    return await page.find(text=text, best_match=True)

def find_nearest_text(page, text):
    return asyncio.run(async_find_nearest_text(page, text))

async def async_find_nearest_guess(page, guess):
    return await page.find(guess, best_match=True)

def find_nearest_guess(page, guess):
    return asyncio.run(async_find_nearest_guess(page, guess))

async def async_find_all_by_css_selector(page, css_selector):
    return await page.select_all(css_selector)

def find_all_by_css_selector(page, css_selector):
    return asyncio.run(async_find_nearest_guess(page, css_selector))

async def async_send_keys_to_element(element, string):
    return await element.send_keys(str(string))

def send_keys_to_element(element, string):
    return asyncio.run(async_send_keys_to_element(element, string))

async def async_click_item(item):
    return await item.mouse_click()

def click_item(item):
    return asyncio.run(async_click_item(item))

async def async_get_site_soup(browser, site, wait=0.5):
    page = await browser.get(site)
    time.sleep(wait)
    content = await page.get_content()
    soup = BeautifulSoup(content, 'html.parser')
    return soup
    
def get_site_soup(browser, site, wait=0.5):
    return asyncio.run(async_get_site_soup(browser, site, wait))

async def open_nodriver(headless=False, proxy=None, tor=False, max_attempts=3):
    """
    Open a browser instance using NoDriver.
    
    Args:
        headless (bool): Whether to run the browser in headless mode.
        proxy (bool): Whether to use a proxy server.
        tor (bool): Whether to route requests through Tor.
        max_attempts (int): Maximum attempts for fetching a proxy.

    Returns:
        browser: The initialized browser instance.
    """
    global hyperSelProxies
    browser_args = ["--start-maximized"]
    browser_args.append(f"--user-agent={general_utilities.generate_random_user_agent()}")

    if tor:
        # print("Routing requests through Tor (127.0.0.1:9050).")
        browser_args.append("--proxy-server=socks5://127.0.0.1:9050")

    elif proxy:
        hyperSelProxies = proxies_utilities.HyperSelProxies()
        print("SLEEPING FOR PROXY...")
        time.sleep(10)
        start = time.time()
        attempts = 0

        while True:
            try:
                proxy_ip = random.choice(hyperSelProxies.current_proxies)
                browser_args.append(f"--proxy-server={proxy_ip}")
                print("GOT PROXY", time.time() - start)
                break
            except Exception as e:
                print(f"Failed to get proxy [ATTEMPT:{attempts}]...")
                time.sleep(10)
                attempts += 1
                if attempts >= max_attempts:
                    raise RuntimeError("Failed to fetch proxy after maximum attempts.")

    browser = await nd.start(
        headless=headless,
        browser_args=browser_args,
        lang="en-US",
    )
    return browser

async def custom_kill_browser(browser):
    general_utilities.kill_process_by_pid(browser._process_pid)
    try:
        hyperSelProxies.stop_threads_and_exit()
    except Exception as e:
        pass
    
def test():
    try:
        # Open the browser
        browser = open_browser(headless=False, proxy=None, tor=True, max_attempts=3)
        
        # Navigate to the site
        page = go_to_site(browser=browser, url="http://check.torproject.org")
        print(page)
        
        # Check the current URL to verify navigation
        current_url = get_current_url(page)
        print(f"Current URL: {current_url}")

        input("WAIT")
    except Exception as e:
        print(f"Error during execution: {e}")
    finally:
        # Custom kill browser if needed
        asyncio.run(custom_kill_browser(browser))

if __name__ == '__main__':
    test()